@@ -9,6 +9,7 @@ pub mod format;
 pub mod genotypes;
 pub mod ids;
 pub mod info;
+mod parser;
 pub mod position;
 pub mod quality_score;
 pub mod reference_bases;
@@ -17,7 +18,8 @@ pub(crate) mod value;
 pub use self::{
     alternate_bases::AlternateBases, builder::Builder, chromosome::Chromosome, field::Field,
     filters::Filters, format::Format, genotypes::Genotypes, ids::Ids, info::Info,
-    position::Position, quality_score::QualityScore, reference_bases::ReferenceBases,
+    parser::ParseError, position::Position, quality_score::QualityScore,
+    reference_bases::ReferenceBases,
 };
 
 #[deprecated(
@@ -26,7 +28,7 @@ pub use self::{
 )]
 pub use self::genotypes::{genotype, Genotype};
 
-use std::{convert::TryFrom, error, fmt, num, str::FromStr};
+use std::{error, fmt, num, str::FromStr};
 
 pub(crate) const MISSING_FIELD: &str = ".";
 pub(crate) const FIELD_DELIMITER: char = '\t';
@@ -46,8 +48,8 @@ pub struct Record {
     ids: Ids,
     reference_bases: ReferenceBases,
     alternate_bases: AlternateBases,
-    quality_score: QualityScore,
-    filters: Filters,
+    quality_score: Option<QualityScore>,
+    filters: Option<Filters>,
     info: Info,
     format: Option<Format>,
     genotypes: Genotypes,
@@ -75,7 +77,6 @@ impl Record {
     /// # Examples
     ///
     /// ```
-    /// # use std::convert::TryFrom;
     /// use noodles_vcf::{self as vcf, record::{Chromosome, Position}};
     ///
     /// let record = vcf::Record::builder()
@@ -91,6 +92,28 @@ impl Record {
         &self.chromosome
     }
 
+    /// Returns a mutable reference to the chromosome.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use noodles_vcf::{self as vcf, record::{Chromosome, Position}};
+    ///
+    /// let mut record = vcf::Record::builder()
+    ///     .set_chromosome("sq0".parse()?)
+    ///     .set_position(Position::try_from(1)?)
+    ///     .set_reference_bases("A".parse()?)
+    ///     .build()?;
+    ///
+    /// *record.chromosome_mut() = "sq1".parse()?;
+    ///
+    /// assert_eq!(record.chromosome().to_string(), "sq1");
+    /// # Ok::<(), Box<dyn std::error::Error>>(())
+    /// ```
+    pub fn chromosome_mut(&mut self) -> &mut Chromosome {
+        &mut self.chromosome
+    }
+
     /// Returns the start position of the reference bases or indicates a telomeric breakend.
     ///
     /// This field is overloaded. If the record represents a telomere, the telomeric breakends are
@@ -102,7 +125,6 @@ impl Record {
     /// # Examples
     ///
     /// ```
-    /// # use std::convert::TryFrom;
     /// use noodles_vcf::{self as vcf, record::Position};
     ///
     /// let record = vcf::Record::builder()
@@ -118,12 +140,37 @@ impl Record {
         self.position
     }
 
+    /// Returns a mutable reference to the start position of the reference bases.
+    ///
+    /// This field is overloaded. If the record represents a telomere, the telomeric breakends are
+    /// set to 0 and _n_ + 1, where _n_ is the length of the chromosome. Otherwise, it is a 1-based
+    /// start position of the reference bases.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use noodles_vcf::{self as vcf, record::Position};
+    ///
+    /// let mut record = vcf::Record::builder()
+    ///     .set_chromosome("sq0".parse()?)
+    ///     .set_position(Position::try_from(8)?)
+    ///     .set_reference_bases("A".parse()?)
+    ///     .build()?;
+    ///
+    /// *record.position_mut() = Position::try_from(13)?;
+    ///
+    /// assert_eq!(i32::from(record.position()), 13);
+    /// # Ok::<(), Box<dyn std::error::Error>>(())
+    /// ```
+    pub fn position_mut(&mut self) -> &mut Position {
+        &mut self.position
+    }
+
     /// Returns a list of IDs of the record.
     ///
     /// # Examples
     ///
     /// ```
-    /// # use std::convert::TryFrom;
     /// use noodles_vcf::{self as vcf, record::Position};
     ///
     /// let record = vcf::Record::builder()
@@ -140,6 +187,29 @@ impl Record {
         &self.ids
     }
 
+    /// Returns a mutable reference to the IDs.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use noodles_vcf::{self as vcf, record::{Ids, Position}};
+    ///
+    /// let mut record = vcf::Record::builder()
+    ///     .set_chromosome("sq0".parse()?)
+    ///     .set_position(Position::try_from(1)?)
+    ///     .set_reference_bases("A".parse()?)
+    ///     .build()?;
+    ///
+    /// let ids: Ids = "nd0".parse()?;
+    /// *record.ids_mut() = ids.clone();
+    ///
+    /// assert_eq!(record.ids(), &ids);
+    /// # Ok::<(), Box<dyn std::error::Error>>(())
+    /// ```
+    pub fn ids_mut(&mut self) -> &mut Ids {
+        &mut self.ids
+    }
+
     /// Returns the reference bases of the record.
     ///
     /// This is a required field and guaranteed to be nonempty.
@@ -147,7 +217,6 @@ impl Record {
     /// # Examples
     ///
     /// ```
-    /// # use std::convert::TryFrom;
     /// use noodles_vcf::{
     ///     self as vcf,
     ///     record::{reference_bases::Base, Position, ReferenceBases},
@@ -174,7 +243,6 @@ impl Record {
     /// # Examples
     ///
     /// ```
-    /// # use std::convert::TryFrom;
     /// use noodles_vcf::{
     ///     self as vcf,
     ///     record::{alternate_bases::Allele, reference_bases::Base, AlternateBases, Position},
@@ -206,7 +274,6 @@ impl Record {
     /// # Examples
     ///
     /// ```
-    /// use std::convert::TryFrom;
     /// use noodles_vcf::{self as vcf, record::{Position, QualityScore}};
     ///
     /// let record = vcf::Record::builder()
@@ -216,13 +283,35 @@ impl Record {
     ///     .set_quality_score(QualityScore::try_from(13.0)?)
     ///     .build()?;
     ///
-    /// assert_eq!(*record.quality_score(), Some(13.0));
+    /// assert_eq!(record.quality_score().map(f32::from), Some(13.0));
     /// # Ok::<(), Box<dyn std::error::Error>>(())
     /// ```
-    pub fn quality_score(&self) -> QualityScore {
+    pub fn quality_score(&self) -> Option<QualityScore> {
         self.quality_score
     }
 
+    /// Returns a mutable reference to the quality score.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use noodles_vcf::{self as vcf, record::{Position, QualityScore}};
+    ///
+    /// let mut record = vcf::Record::builder()
+    ///     .set_chromosome("sq0".parse()?)
+    ///     .set_position(Position::try_from(1)?)
+    ///     .set_reference_bases("A".parse()?)
+    ///     .build()?;
+    ///
+    /// *record.quality_score_mut() = QualityScore::try_from(13.0).map(Some)?;
+    ///
+    /// assert_eq!(record.quality_score().map(f32::from), Some(13.0));
+    /// # Ok::<(), Box<dyn std::error::Error>>(())
+    /// ```
+    pub fn quality_score_mut(&mut self) -> &mut Option<QualityScore> {
+        &mut self.quality_score
+    }
+
     /// Returns the filters of the record.
     ///
     /// The filters can either be pass (`PASS`), a list of filter names that caused the record to
@@ -230,9 +319,6 @@ impl Record {
     ///
     /// # Examples
     ///
-    /// ```
-    /// # use std::convert::TryFrom;
-    /// use noodles_vcf::{self as vcf, record::{Filters, Position}};
     ///
     /// let record = vcf::Record::builder()
     ///     .set_chromosome("sq0".parse()?)
@@ -241,11 +327,33 @@ impl Record {
     ///     .set_filters(Filters::Pass)
     ///     .build()?;
     ///
-    /// assert_eq!(record.filters(), &Filters::Pass);
+    /// assert_eq!(record.filters(), Some(&Filters::Pass));
     /// # Ok::<(), Box<dyn std::error::Error>>(())
     /// ```
-    pub fn filters(&self) -> &Filters {
-        &self.filters
+    pub fn filters(&self) -> Option<&Filters> {
+        self.filters.as_ref()
+    }
+
+    /// Returns a mutable reference to the filters.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use noodles_vcf::{self as vcf, record::{Filters, Position}};
+    ///
+    /// let mut record = vcf::Record::builder()
+    ///     .set_chromosome("sq0".parse()?)
+    ///     .set_position(Position::try_from(1)?)
+    ///     .set_reference_bases("A".parse()?)
+    ///     .build()?;
+    ///
+    /// *record.filters_mut() = Some(Filters::Pass);
+    ///
+    /// assert_eq!(record.filters(), Some(&Filters::Pass));
+    /// # Ok::<(), Box<dyn std::error::Error>>(())
+    /// ```
+    pub fn filters_mut(&mut self) -> &mut Option<Filters> {
+        &mut self.filters
     }
 
     /// Returns the addition information of the record.
@@ -253,7 +361,6 @@ impl Record {
     /// # Examples
     ///
     /// ```
-    /// # use std::convert::TryFrom;
     /// use noodles_vcf::{
     ///     self as vcf,
     ///     record::{info::{field::{Key, Value}, Field}, Info, Position},
@@ -279,12 +386,45 @@ impl Record {
         &self.info
     }
 
+    /// Returns a mutable reference to the additional info fields.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use noodles_vcf::{
+    ///     self as vcf,
+    ///     record::{info::{field::{Key, Value}, Field}, Info, Position},
+    /// };
+    ///
+    /// let mut record = vcf::Record::builder()
+    ///     .set_chromosome("sq0".parse()?)
+    ///     .set_position(Position::try_from(1)?)
+    ///     .set_reference_bases("A".parse()?)
+    ///     .set_alternate_bases("C".parse()?)
+    ///     .set_info("NS=3;AF=0.5".parse()?)
+    ///     .build()?;
+    ///
+    /// let dp = Field::new(Key::TotalDepth, Value::Integer(13));
+    /// record.info_mut().insert(dp.clone());
+    ///
+    /// let expected = Info::try_from(vec![
+    ///     Field::new(Key::SamplesWithDataCount, Value::Integer(3)),
+    ///     Field::new(Key::AlleleFrequencies, Value::FloatArray(vec![0.5])),
+    ///     dp,
+    /// ])?;
+    ///
+    /// assert_eq!(record.info(), &expected);
+    /// # Ok::<(), Box<dyn std::error::Error>>(())
+    /// ```
+    pub fn info_mut(&mut self) -> &mut Info {
+        &mut self.info
+    }
+
     /// Returns the format of the genotypes of the record.
     ///
     /// # Examples
     ///
     /// ```
-    /// # use std::convert::TryFrom;
     /// use noodles_vcf::{
     ///     self as vcf,
     ///     record::{
@@ -319,7 +459,6 @@ impl Record {
     /// # Examples
     ///
     /// ```
-    /// # use std::convert::TryFrom;
     /// use noodles_vcf::{
     ///     self as vcf,
     ///     record::{
@@ -401,7 +540,6 @@ impl Record {
     /// ## From the `END` INFO field value
     ///
     /// ```
-    /// # use std::convert::TryFrom;
     /// use noodles_vcf::{self as vcf, record::{Chromosome, Position}};
     ///
     /// let record = vcf::Record::builder()
@@ -418,7 +556,6 @@ impl Record {
     /// ## Calculated using the start position and reference bases length
     ///
     /// ```
-    /// # use std::convert::TryFrom;
     /// use noodles_vcf::{self as vcf, record::{Chromosome, Position}};
     ///
     /// let record = vcf::Record::builder()
@@ -456,17 +593,28 @@ impl fmt::Display for Record {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(
             f,
-            "{chrom}\t{pos}\t{id}\t{ref}\t{alt}\t{qual}\t{filter}\t{info}",
+            "{chrom}\t{pos}\t{id}\t{ref}\t{alt}",
             chrom = self.chromosome(),
             pos = i32::from(self.position()),
             id = self.ids(),
             r#ref = self.reference_bases(),
             alt = self.alternate_bases(),
-            qual = self.quality_score(),
-            filter = self.filters(),
-            info = self.info(),
         )?;
 
+        if let Some(quality_score) = self.quality_score() {
+            write!(f, "\t{}", quality_score)?;
+        } else {
+            write!(f, "\t{}", MISSING_FIELD)?;
+        }
+
+        if let Some(filters) = self.filters() {
+            write!(f, "\t{}", filters)?;
+        } else {
+            write!(f, "\t{}", MISSING_FIELD)?;
+        }
+
+        write!(f, "\t{}", self.info())?;
+
         if let Some(format) = self.format() {
             write!(f, "\t{}", format)?;
             write!(f, "\t{}", self.genotypes())?;
@@ -476,125 +624,16 @@ impl fmt::Display for Record {
     }
 }
 
-/// An error returned when a raw VCF record fails to parse.
-#[derive(Clone, Debug, Eq, PartialEq)]
-pub enum ParseError {
-    /// A field is missing.
-    MissingField(Field),
-    /// The chromosome is invalid.
-    InvalidChromosome(chromosome::ParseError),
-    /// The position is invalid.
-    InvalidPosition(position::ParseError),
-    /// The ID is invalid.
-    InvalidIds(ids::ParseError),
-    /// The reference bases are invalid.
-    InvalidReferenceBases(reference_bases::ParseError),
-    /// The alternate bases are invalid.
-    InvalidAlternateBases(alternate_bases::ParseError),
-    /// The quality score is invalid.
-    InvalidQualityScore(quality_score::ParseError),
-    /// A filter is invalid.
-    InvalidFilters(filters::ParseError),
-    /// The info is invalid.
-    InvalidInfo(info::ParseError),
-    /// The format is invalid.
-    InvalidFormat(format::ParseError),
-    /// A genotype is invalid.
-    InvalidGenotype(genotypes::genotype::ParseError),
-}
-
-impl error::Error for ParseError {}
-
-impl fmt::Display for ParseError {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        match self {
-            Self::MissingField(field) => write!(f, "missing field: {}", field),
-            Self::InvalidChromosome(e) => write!(f, "invalid chromosome: {}", e),
-            Self::InvalidPosition(e) => write!(f, "invalid position: {}", e),
-            Self::InvalidIds(e) => write!(f, "invalid IDs: {}", e),
-            Self::InvalidReferenceBases(e) => write!(f, "invalid reference bases: {}", e),
-            Self::InvalidAlternateBases(e) => write!(f, "invalid alternate bases: {}", e),
-            Self::InvalidQualityScore(e) => write!(f, "invalid quality score: {}", e),
-            Self::InvalidFilters(e) => write!(f, "invalid filters: {}", e),
-            Self::InvalidInfo(e) => write!(f, "invalid info: {}", e),
-            Self::InvalidFormat(e) => write!(f, "invalid format: {}", e),
-            Self::InvalidGenotype(e) => write!(f, "invalid genotype: {}", e),
-        }
-    }
-}
-
 impl FromStr for Record {
     type Err = ParseError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        let mut fields = s.split(FIELD_DELIMITER);
-
-        let chrom = parse_string(&mut fields, Field::Chromosome)
-            .and_then(|s| s.parse().map_err(ParseError::InvalidChromosome))?;
-
-        let pos = parse_string(&mut fields, Field::Position)
-            .and_then(|s| s.parse().map_err(ParseError::InvalidPosition))?;
-
-        let ids = parse_string(&mut fields, Field::Ids)
-            .and_then(|s| s.parse().map_err(ParseError::InvalidIds))?;
-
-        let r#ref = parse_string(&mut fields, Field::ReferenceBases)
-            .and_then(|s| s.parse().map_err(ParseError::InvalidReferenceBases))?;
-
-        let alt = parse_string(&mut fields, Field::AlternateBases)
-            .and_then(|s| s.parse().map_err(ParseError::InvalidAlternateBases))?;
-
-        let qual = parse_string(&mut fields, Field::QualityScore)
-            .and_then(|s| s.parse().map_err(ParseError::InvalidQualityScore))?;
-
-        let filter = parse_string(&mut fields, Field::Filters)
-            .and_then(|s| s.parse().map_err(ParseError::InvalidFilters))?;
-
-        let info = parse_string(&mut fields, Field::Info)
-            .and_then(|s| s.parse().map_err(ParseError::InvalidInfo))?;
-
-        let format = match fields.next() {
-            Some(s) => s.parse().map(Some).map_err(ParseError::InvalidFormat)?,
-            None => None,
-        };
-
-        let genotypes = format
-            .as_ref()
-            .map(|f| {
-                fields
-                    .map(|s| genotypes::Genotype::from_str_format(s, f))
-                    .collect::<Result<_, _>>()
-                    .map_err(ParseError::InvalidGenotype)
-            })
-            .unwrap_or_else(|| Ok(Vec::new()))?
-            .into();
-
-        Ok(Self {
-            chromosome: chrom,
-            position: pos,
-            ids,
-            reference_bases: r#ref,
-            alternate_bases: alt,
-            quality_score: qual,
-            filters: filter,
-            info,
-            format,
-            genotypes,
-        })
+        parser::parse(s)
     }
 }
 
-fn parse_string<'a, I>(fields: &mut I, field: Field) -> Result<&'a str, ParseError>
-where
-    I: Iterator<Item = &'a str>,
-{
-    fields.next().ok_or(ParseError::MissingField(field))
-}
-
 #[cfg(test)]
 mod tests {
-    use std::convert::TryFrom;
-
     use super::*;
 
     #[test]
@@ -659,70 +698,4 @@ mod tests {
 
         Ok(())
     }
-
-    #[test]
-    fn test_from_str() -> Result<(), ParseError> {
-        use alternate_bases::Allele;
-        use reference_bases::Base;
-
-        let s = "chr1\t13\tnd0\tATCG\tA\t5.8\tPASS\tSVTYPE=DEL";
-        let record: Record = s.parse()?;
-
-        assert!(matches!(record.chromosome(), Chromosome::Name(name) if name == "chr1"));
-
-        assert_eq!(i32::from(record.position()), 13);
-
-        let ids = record.ids();
-        assert_eq!(ids.len(), 1);
-        assert!(ids.contains("nd0"));
-
-        let reference_bases = [Base::A, Base::T, Base::C, Base::G];
-        assert_eq!(&record.reference_bases()[..], &reference_bases[..]);
-
-        let alternate_bases = [Allele::Bases(vec![Base::A])];
-        assert_eq!(&record.alternate_bases()[..], &alternate_bases[..]);
-
-        assert_eq!(*record.quality_score(), Some(5.8));
-        assert_eq!(record.filters(), &Filters::Pass);
-        assert_eq!(record.info().len(), 1);
-        assert!(record.format().is_none());
-        assert!(record.genotypes().is_empty());
-
-        Ok(())
-    }
-
-    #[test]
-    fn test_from_str_with_genotype_info() -> Result<(), Box<dyn std::error::Error>> {
-        use genotypes::genotype;
-
-        let s = "chr1\t13\tnd0\tATCG\tA\t5.8\tPASS\tSVTYPE=DEL\tGT:GQ\t0|1:13";
-        let record: Record = s.parse()?;
-
-        let expected = Format::try_from(vec![
-            genotype::field::Key::Genotype,
-            genotype::field::Key::ConditionalGenotypeQuality,
-        ])?;
-
-        assert_eq!(record.format(), Some(&expected));
-
-        let genotypes = record.genotypes();
-
-        assert_eq!(genotypes.len(), 1);
-
-        let actual: Vec<_> = genotypes[0].values().cloned().collect();
-        let expected = vec![
-            genotype::Field::new(
-                genotype::field::Key::Genotype,
-                Some(genotype::field::Value::String(String::from("0|1"))),
-            ),
-            genotype::Field::new(
-                genotype::field::Key::ConditionalGenotypeQuality,
-                Some(genotype::field::Value::Integer(13)),
-            ),
-        ];
-
-        assert_eq!(actual, expected);
-
-        Ok(())
-    }
 }
