@@ -1,6 +1,6 @@
 //! VCF record builder.
 
-use std::{convert::TryFrom, error, fmt};
+use std::{error, fmt};
 
 use super::{
     genotypes::Genotype, reference_bases::Base, AlternateBases, Chromosome, Filters, Format,
@@ -15,8 +15,8 @@ pub struct Builder {
     ids: Ids,
     reference_bases: Vec<Base>,
     alternate_bases: AlternateBases,
-    quality_score: QualityScore,
-    filters: Filters,
+    quality_score: Option<QualityScore>,
+    filters: Option<Filters>,
     info: Info,
     format: Option<Format>,
     genotypes: Genotypes,
@@ -51,7 +51,6 @@ impl Builder {
     /// # Examples
     ///
     /// ```
-    /// # use std::convert::TryFrom;
     /// use noodles_vcf::{self as vcf, record::{Chromosome, Position}};
     ///
     /// let record = vcf::Record::builder()
@@ -73,7 +72,6 @@ impl Builder {
     /// # Examples
     ///
     /// ```
-    /// # use std::convert::TryFrom;
     /// use noodles_vcf::{self as vcf, record::Position};
     ///
     /// let record = vcf::Record::builder()
@@ -95,7 +93,6 @@ impl Builder {
     /// # Examples
     ///
     /// ```
-    /// # use std::convert::TryFrom;
     /// use noodles_vcf::{self as vcf, record::Position};
     ///
     /// let record = vcf::Record::builder()
@@ -118,7 +115,6 @@ impl Builder {
     /// # Examples
     ///
     /// ```
-    /// # use std::convert::TryFrom;
     /// use noodles_vcf::{
     ///     self as vcf,
     ///     record::{reference_bases::Base, Position, ReferenceBases},
@@ -147,7 +143,6 @@ impl Builder {
     /// # Examples
     ///
     /// ```
-    /// # use std::convert::TryFrom;
     /// use noodles_vcf::{
     ///     self as vcf,
     ///     record::{reference_bases::Base, Position, ReferenceBases},
@@ -175,7 +170,6 @@ impl Builder {
     /// # Examples
     ///
     /// ```
-    /// # use std::convert::TryFrom;
     /// use noodles_vcf::{
     ///     self as vcf,
     ///     record::{alternate_bases::Allele, reference_bases::Base, AlternateBases, Position},
@@ -204,7 +198,6 @@ impl Builder {
     /// # Examples
     ///
     /// ```
-    /// # use std::convert::TryFrom;
     /// use noodles_vcf::{self as vcf, record::{Position, QualityScore}};
     ///
     /// let record = vcf::Record::builder()
@@ -214,11 +207,11 @@ impl Builder {
     ///     .set_quality_score(QualityScore::try_from(13.0)?)
     ///     .build()?;
     ///
-    /// assert_eq!(*record.quality_score(), Some(13.0));
+    /// assert_eq!(record.quality_score().map(f32::from), Some(13.0));
     /// # Ok::<(), Box<dyn std::error::Error>>(())
     /// ```
     pub fn set_quality_score(mut self, quality_score: QualityScore) -> Self {
-        self.quality_score = quality_score;
+        self.quality_score = Some(quality_score);
         self
     }
 
@@ -227,7 +220,6 @@ impl Builder {
     /// # Examples
     ///
     /// ```
-    /// # use std::convert::TryFrom;
     /// use noodles_vcf::{self as vcf, record::{Filters, Position}};
     ///
     /// let record = vcf::Record::builder()
@@ -237,11 +229,11 @@ impl Builder {
     ///     .set_filters(Filters::Pass)
     ///     .build()?;
     ///
-    /// assert_eq!(record.filters(), &Filters::Pass);
+    /// assert_eq!(record.filters(), Some(&Filters::Pass));
     /// # Ok::<(), Box<dyn std::error::Error>>(())
     /// ```
     pub fn set_filters(mut self, filters: Filters) -> Self {
-        self.filters = filters;
+        self.filters = Some(filters);
         self
     }
 
@@ -250,7 +242,6 @@ impl Builder {
     /// # Examples
     ///
     /// ```
-    /// # use std::convert::TryFrom;
     /// use noodles_vcf::{
     ///     self as vcf,
     ///     record::{info::{field::{Key, Value}, Field}, Info, Position},
@@ -282,7 +273,6 @@ impl Builder {
     /// # Examples
     ///
     /// ```
-    /// # use std::convert::TryFrom;
     /// use noodles_vcf::{
     ///     self as vcf,
     ///     record::{
@@ -318,7 +308,6 @@ impl Builder {
     /// # Examples
     ///
     /// ```
-    /// # use std::convert::TryFrom;
     /// use noodles_vcf::{
     ///     self as vcf,
     ///     record::{
@@ -361,7 +350,6 @@ impl Builder {
     /// # Examples
     ///
     /// ```
-    /// # use std::convert::TryFrom;
     /// use noodles_vcf::{
     ///     self as vcf,
     ///     record::{
@@ -436,7 +424,7 @@ mod tests {
         assert!(record.reference_bases.is_empty());
         assert!(record.alternate_bases.is_empty());
         assert!(record.quality_score.is_none());
-        assert!(matches!(record.filters, Filters::Missing));
+        assert!(record.filters.is_none());
         assert!(record.info.is_empty());
         assert!(record.format.is_none());
         assert!(record.genotypes.is_empty());
